process.env.NODE_TLS_REJECT_UNAUTHORIZED = '0'
process.env.SUPPRESS_NO_CONFIG_WARNING = true

const { cloneDeep } = require('lodash')
const fixturesService = require('pubsweet-component-fixture-service')

const { fixtures, Model } = fixturesService
const { Collection } = require('../Helper')

describe('Collection helper', () => {
  let testFixtures = {}
  let models
  beforeEach(() => {
    testFixtures = cloneDeep(fixtures)
    models = Model.build(testFixtures)
  })

  describe('getAllCollectionFragments', () => {
    it('should return an array with all collection fragments', async () => {
      const { collection } = testFixtures.collections
      const collectionHelper = new Collection({ collection })
      const FragmentModel = models.Fragment

      const numberOfFragments = collection.fragments.length
      const allCollectionFragments = await collectionHelper.getAllCollectionFragments(
        {
          FragmentModel,
        },
      )

      expect(allCollectionFragments).toHaveLength(numberOfFragments)
      expect(typeof allCollectionFragments[0]).toBe('object')
      expect(allCollectionFragments[0].type).toBe('fragment')
    })
    it('should return an empty array when no fragments exist', async () => {
      const { noFragmentsCollection } = testFixtures.collections
      const collectionHelper = new Collection({ noFragmentsCollection })
      const FragmentModel = models.Fragment

      const allCollectionFragments = await collectionHelper.getAllCollectionFragments(
        {
          FragmentModel,
        },
      )

      expect(allCollectionFragments).toHaveLength(0)
      expect(typeof allCollectionFragments[0]).toBe('undefined')
    })
  })

  describe('getReviewerNumber', () => {
    it('should assign reviewer number 1 on invitation if no other reviewer numbers exist', async () => {
      const { collection } = testFixtures.collections
      const { reviewer } = testFixtures.users
      const collectionHelper = new Collection({ collection })
      const FragmentModel = models.Fragment

      const reviewerNumber = await collectionHelper.getReviewerNumber({
        userId: reviewer.id,
        FragmentModel,
      })

      expect(reviewerNumber).toBe(1)
    })
    it('should assign next reviewer number on invitation if another reviewer numbers exist', async () => {
      const { collectionReviewCompleted } = testFixtures.collections
      const { reviewer } = testFixtures.users
      const collectionHelper = new Collection({
        collection: collectionReviewCompleted,
      })
      const FragmentModel = models.Fragment

      const reviewerNumber = await collectionHelper.getReviewerNumber({
        userId: reviewer.id,
        FragmentModel,
      })

      expect(reviewerNumber).toBe(3)
    })
    it('should keep reviewer number across fragment versions', async () => {
      const { oneReviewedFragmentCollection } = testFixtures.collections
      const { answerReviewer } = testFixtures.users
      const collectionHelper = new Collection({
        collection: oneReviewedFragmentCollection,
      })
      const FragmentModel = models.Fragment

      const reviewerNumber = await collectionHelper.getReviewerNumber({
        userId: answerReviewer.id,
        FragmentModel,
      })

      expect(reviewerNumber).toBe(2)
    })
    it('should assign next reviewer number across fragment versions', async () => {
      const { oneReviewedFragmentCollection } = testFixtures.collections
      const { reviewer } = testFixtures.users
      const collectionHelper = new Collection({
        collection: oneReviewedFragmentCollection,
      })
      const FragmentModel = models.Fragment

      const reviewerNumber = await collectionHelper.getReviewerNumber({
        userId: reviewer.id,
        FragmentModel,
      })

      expect(reviewerNumber).toBe(3)
    })
  })
})
